@@ -77,15 +77,25 @@ module GitHubPages
         185.199.111.153
       ).freeze
 
+      CURRENT_IPV6_ADDRESSES = %w(
+        2606:50c0:8000::153
+        2606:50c0:8001::153
+        2606:50c0:8002::153
+        2606:50c0:8003::153
+      ).freeze
+
+      CURRENT_IP_ADDRESSES_ALL =
+        (CURRENT_IP_ADDRESSES + CURRENT_IPV6_ADDRESSES).freeze
+
       HASH_METHODS = %i[
         host uri nameservers dns_resolves? proxied? cloudflare_ip?
-        fastly_ip? old_ip_address? a_record? cname_record?
-        mx_records_present? valid_domain? apex_domain? should_be_a_record?
-        cname_to_github_user_domain? cname_to_pages_dot_github_dot_com?
-        cname_to_fastly? pointed_to_github_pages_ip?
-        non_github_pages_ip_present? pages_domain?
+        fastly_ip? old_ip_address? a_record? aaaa_record? a_record_present? aaaa_record_present?
+        cname_record? mx_records_present? valid_domain? apex_domain?
+        should_be_a_record? cname_to_github_user_domain? cname_to_domain_to_pages?
+        cname_to_pages_dot_github_dot_com? cname_to_fastly?
+        pointed_to_github_pages_ip? non_github_pages_ip_present? pages_domain?
         served_by_pages? valid? reason valid_domain? https?
-        enforces_https? https_error https_eligible? caa_error
+        enforces_https? https_error https_eligible? caa_error dns_zone_soa? dns_zone_ns?
       ].freeze
 
       def self.redundant(host)
@@ -100,22 +110,24 @@ module GitHubPages
         @host = normalize_host(host)
         @nameservers = nameservers
         @resolver = GitHubPages::HealthCheck::Resolver.new(self.host,
-          :nameservers => nameservers)
+                                                           :nameservers => nameservers)
       end
 
       # Runs all checks, raises an error if invalid
+      # rubocop:disable Metrics/AbcSize
       def check!
-        raise Errors::InvalidDomainError, :domain => self unless valid_domain?
-        raise Errors::InvalidDNSError, :domain => self    unless dns_resolves?
-        raise Errors::DeprecatedIPError, :domain => self if deprecated_ip?
+        raise Errors::InvalidDomainError.new :domain => self unless valid_domain?
+        raise Errors::InvalidDNSError.new :domain => self    unless dns_resolves?
+        raise Errors::DeprecatedIPError.new :domain => self  if deprecated_ip?
         return true if proxied?
-        raise Errors::InvalidARecordError, :domain => self    if invalid_a_record?
-        raise Errors::InvalidCNAMEError, :domain => self      if invalid_cname?
-        raise Errors::InvalidAAAARecordError, :domain => self if invalid_aaaa_record?
-        raise Errors::NotServedByPagesError, :domain => self  unless served_by_pages?
+        raise Errors::InvalidARecordError.new :domain => self    if invalid_a_record?
+        raise Errors::InvalidCNAMEError.new :domain => self      if invalid_cname?
+        raise Errors::InvalidAAAARecordError.new :domain => self if invalid_aaaa_record?
+        raise Errors::NotServedByPagesError.new :domain => self  unless served_by_pages?
 
         true
       end
+      # rubocop:enable Metrics/AbcSize
 
       def deprecated_ip?
         return @deprecated_ip if defined? @deprecated_ip
@@ -126,14 +138,13 @@ module GitHubPages
       def invalid_aaaa_record?
         return @invalid_aaaa_record if defined? @invalid_aaaa_record
 
-        @invalid_aaaa_record = (valid_domain? && should_be_a_record? &&
-                                aaaa_record_present?)
+        @invalid_aaaa_record = (valid_domain? && aaaa_record_present? && !should_be_a_record?)
       end
 
       def invalid_a_record?
         return @invalid_a_record if defined? @invalid_a_record
 
-        @invalid_a_record = (valid_domain? && a_record? && !should_be_a_record?)
+        @invalid_a_record = (valid_domain? && a_record_present? && !should_be_a_record?)
       end
 
       def invalid_cname?
@@ -149,7 +160,7 @@ module GitHubPages
       end
 
       # Is this a valid domain that PublicSuffix recognizes?
-      # Used as an escape hatch to prevent false positives on DNS checkes
+      # Used as an escape hatch to prevent false positives on DNS checks
       def valid_domain?
         return @valid if defined? @valid
 
@@ -159,10 +170,13 @@ module GitHubPages
                                      :ignore_private => true)
       end
 
-      # Is this domain an apex domain, meaning a CNAME would be innapropriate
+      # Is this domain an apex domain, meaning a CNAME would be inappropriate
       def apex_domain?
         return @apex_domain if defined?(@apex_domain)
-        return unless valid_domain?
+
+        return false unless valid_domain?
+
+        return true if dns_zone_soa? && dns_zone_ns?
 
         # PublicSuffix.domain pulls out the apex-level domain name.
         # E.g. PublicSuffix.domain("techblog.netflix.com") # => "netflix.com"
@@ -175,6 +189,30 @@ module GitHubPages
                             :ignore_private => true) == unicode_host
       end
 
+      #
+      # Does the domain have an associated SOA record?
+      #
+      def dns_zone_soa?
+        return @soa_records if defined?(@soa_records)
+        return false unless dns?
+
+        @soa_records = dns.any? do |answer|
+          answer.type == Dnsruby::Types::SOA && answer.name.to_s == host
+        end
+      end
+
+      #
+      # Does the domain have associated NS records?
+      #
+      def dns_zone_ns?
+        return @ns_records if defined?(@ns_records)
+        return false unless dns?
+
+        @ns_records = dns.any? do |answer|
+          answer.type == Dnsruby::Types::NS && answer.name.to_s == host
+        end
+      end
+
       # Should the domain use an A record?
       def should_be_a_record?
         !pages_io_domain? && (apex_domain? || mx_records_present?)
@@ -184,20 +222,20 @@ module GitHubPages
         !should_be_a_record?
       end
 
-      # Is the domain's first response an A record to a valid GitHub Pages IP?
+      # Is the domain's first response an A or AAAA record to a valid GitHub Pages IP?
       def pointed_to_github_pages_ip?
-        a_record? && CURRENT_IP_ADDRESSES.include?(dns.first.address.to_s)
+        return false unless address_record?
+
+        CURRENT_IP_ADDRESSES_ALL.include?(dns.first.address.to_s.downcase)
       end
 
-      # Are any of the domain's A records pointing elsewhere?
+      # Are any of the domain's A or AAAA records pointing elsewhere?
       def non_github_pages_ip_present?
         return unless dns?
 
-        a_records = dns.select { |answer| answer.type == Dnsruby::Types::A }
-
-        a_records.any? { |answer| !github_pages_ip?(answer.address.to_s) }
-
-        false
+        dns
+          .select { |a| Dnsruby::Types::A == a.type || Dnsruby::Types::AAAA == a.type }
+          .any? { |a| !github_pages_ip?(a.address.to_s) }
       end
 
       # Is the domain's first response a CNAME to a pages domain?
@@ -205,10 +243,24 @@ module GitHubPages
         cname? && !cname_to_pages_dot_github_dot_com? && cname.pages_domain?
       end
 
+      # Check if the CNAME points to a Domain that points to pages
+      # e.g. CNAME -> Domain -> Pages
+      # rubocop:disable Metrics/AbcSize
+      def cname_to_domain_to_pages?
+        return false unless dns?
+
+        a_record_to_pages = dns.select { |d| d.type == Dnsruby::Types::A && d.name.to_s == host }.first
+
+        return false unless a_record_to_pages && cname? && !cname_to_pages_dot_github_dot_com? && @www_cname
+
+        CURRENT_IP_ADDRESSES.include?(a_record_to_pages.address.to_s.downcase)
+      end
+      # rubocop:enable Metrics/AbcSize
+
       # Is the given domain a CNAME to pages.github.(io|com)
       # instead of being CNAME'd to the user's subdomain?
       #
-      # domain - the domain to check, generaly the target of a cname
+      # domain - the domain to check, generally the target of a cname
       def cname_to_pages_dot_github_dot_com?
         cname? && cname.pages_dot_github_dot_com?
       end
@@ -235,7 +287,7 @@ module GitHubPages
 
       # Is this domain owned by GitHub?
       def github_domain?
-        !!host.downcase.end_with?("github.com")
+        host.downcase.eql?("github.com") || host.downcase.end_with?(".github.com")
       end
 
       # Is the host our Fastly CNAME?
@@ -266,6 +318,7 @@ module GitHubPages
         return true if cloudflare_ip?
         return false if pointed_to_github_pages_ip?
         return false if cname_to_github_user_domain?
+        return false if cname_to_domain_to_pages?
         return false if cname_to_pages_dot_github_dot_com?
         return false if cname_to_fastly? || fastly_ip?
 
@@ -276,7 +329,9 @@ module GitHubPages
         Dnsruby::Types::A,
         Dnsruby::Types::AAAA,
         Dnsruby::Types::CNAME,
-        Dnsruby::Types::MX
+        Dnsruby::Types::MX,
+        Dnsruby::Types::NS,
+        Dnsruby::Types::SOA
       ].freeze
 
       # Returns an array of DNS answers
@@ -314,15 +369,32 @@ module GitHubPages
 
       # Is this domain's first response an A record?
       def a_record?
+        return @is_a_record if defined?(@is_a_record)
         return unless dns?
 
-        dns.first.type == Dnsruby::Types::A
+        @is_a_record = Dnsruby::Types::A == dns.first.type
       end
 
+      # Is this domain's first response an AAAA record?
+      def aaaa_record?
+        return @is_aaaa_record if defined?(@is_aaaa_record)
+        return unless dns?
+
+        @is_aaaa_record = Dnsruby::Types::AAAA == dns.first.type
+      end
+
+      # Does this domain has an A record setup (not necessarily as the first record)?
+      def a_record_present?
+        return unless dns?
+
+        dns.any? { |answer| answer.type == Dnsruby::Types::A && answer.name.to_s == host }
+      end
+
+      # Does this domain has an AAAA record setup (not necessarily as the first record)?
       def aaaa_record_present?
         return unless dns?
 
-        dns.any? { |answer| answer.type == Dnsruby::Types::AAAA }
+        dns.any? { |answer| answer.type == Dnsruby::Types::AAAA && answer.name.to_s == host }
       end
 
       # Is this domain's first response a CNAME record?
@@ -337,12 +409,21 @@ module GitHubPages
       # The domain to which this domain's CNAME resolves
       # Returns nil if the domain is not a CNAME
       def cname
+        return unless dns?
+
         cnames = dns.take_while { |answer| answer.type == Dnsruby::Types::CNAME }
         return if cnames.empty?
 
+        www_cname(cnames.last)
         @cname ||= Domain.new(cnames.last.cname.to_s)
       end
 
+      # Check if we have a 'www.' CNAME that matches the domain
+      def www_cname(cname)
+        @www_cname ||= cname.name.to_s.start_with?("www.") &&
+          cname.name.to_s.end_with?(cname.domainname.to_s)
+      end
+
       def mx_records_present?
         return unless dns?
 
@@ -354,11 +435,10 @@ module GitHubPages
         return unless dns_resolves?
 
         @served_by_pages = begin
-          return false unless response.mock? || response.return_code == :ok
           return true if response.headers["Server"] == "GitHub.com"
 
           # Typhoeus mangles the case of the header, compare insensitively
-          response.headers.any? { |k, _v| k =~ /X-GitHub-Request-Id/i }
+          response.headers.any? { |k, _v| k.downcase == "x-github-request-id" }
         end
       end
 
@@ -391,12 +471,12 @@ module GitHubPages
       def https_eligible?
         # Can't have any IP's which aren't GitHub's present.
         return false if non_github_pages_ip_present?
-        # Can't have any AAAA records present
-        return false if aaaa_record_present?
-        # Must be a CNAME or point to our IPs.
 
-        # Only check the one domain if a CNAME. Don't check the parent domain.
-        return true if cname_to_github_user_domain?
+        # Can't have underscores in the domain name (Let's Encrypt does not allow it)
+        return false if host.include?("_")
+
+        # Must be a CNAME or point to our IPs.
+        return true if cname_to_github_user_domain? || cname_to_domain_to_pages?
 
         # Check CAA records for the full domain and its parent domain.
         pointed_to_github_pages_ip? && caa.lets_encrypt_allowed?
@@ -404,27 +484,34 @@ module GitHubPages
 
       # Any errors querying CAA records
       def caa_error
-        return nil unless caa.errored?
+        return nil unless caa&.errored?
 
         caa.error.class.name
       end
 
       private
 
+      def address_record?
+        a_record? || aaaa_record?
+      end
+
       def caa
-        @caa ||= GitHubPages::HealthCheck::CAA.new(host, :nameservers => nameservers)
+        @caa ||= GitHubPages::HealthCheck::CAA.new(
+          :host => cname&.host || host,
+          :nameservers => nameservers
+        )
       end
 
       # The domain's response to HTTP(S) requests, following redirects
       def response
         return @response if defined? @response
 
-        @response = Typhoeus.head(uri, TYPHOEUS_OPTIONS)
+        @response = Typhoeus.head(uri, GitHubPages::HealthCheck.typhoeus_options)
 
         # Workaround for webmock not playing nicely with Typhoeus redirects
         # See https://github.com/bblimke/webmock/issues/237
         if @response.mock? && @response.headers["Location"]
-          @response = Typhoeus.head(response.headers["Location"], TYPHOEUS_OPTIONS)
+          @response = Typhoeus.head(response.headers["Location"], GitHubPages::HealthCheck.typhoeus_options)
         end
 
         @response
@@ -432,13 +519,13 @@ module GitHubPages
 
       # The domain's response to HTTP requests, without following redirects
       def http_response
-        options = TYPHOEUS_OPTIONS.merge(:followlocation => false)
+        options = GitHubPages::HealthCheck.typhoeus_options.merge(:followlocation => false)
         @http_response ||= Typhoeus.head(uri(:scheme => "http"), options)
       end
 
       # The domain's response to HTTPS requests, without following redirects
       def https_response
-        options = TYPHOEUS_OPTIONS.merge(:followlocation => false)
+        options = GitHubPages::HealthCheck.typhoeus_options.merge(:followlocation => false)
         @https_response ||= Typhoeus.head(uri(:scheme => "https"), options)
       end
 
@@ -482,10 +569,12 @@ module GitHubPages
       def cdn_ip?(cdn)
         return unless dns?
 
-        a_records = dns.select { |answer| answer.type == Dnsruby::Types::A }
-        return false if !a_records || a_records.empty?
+        address_records = dns.select do |answer|
+          Dnsruby::Types::A == answer.type || Dnsruby::Types::AAAA == answer.type
+        end
+        return false if !address_records || address_records.empty?
 
-        a_records.all? do |answer|
+        address_records.all? do |answer|
           cdn.controls_ip?(answer.address)
         end
       end
@@ -495,7 +584,7 @@ module GitHubPages
       end
 
       def github_pages_ip?(ip_addr)
-        CURRENT_IP_ADDRESSES.include?(ip_addr)
+        CURRENT_IP_ADDRESSES_ALL.include?(ip_addr&.to_s&.downcase)
       end
     end
   end
