@@ -4,7 +4,7 @@ module GitHubPages
   module HealthCheck
     module Errors
       class BuildError < GitHubPages::HealthCheck::Error
-        DOCUMENTATION_PATH = "/articles/troubleshooting-jekyll-builds/".freeze
+        DOCUMENTATION_PATH = "/articles/troubleshooting-jekyll-builds/"
         LOCAL_ONLY = true
       end
     end
